@@ -24,8 +24,7 @@
 #include <sys/stat.h>
 #include <stdio.h>
 #include <string.h>
-#include <asm/types.h>
-#include <linux/quota.h>
+#include <sys/quota.h>
 #include <stdarg.h>
 #include <sys/file.h>
 #include <mntent.h>
@@ -39,11 +38,19 @@
 #include <ext2fs/ext2fs.h>
 #endif
 
+#include "bylabel.h"
+
+#include "dqblk.h"
+
+int need_dquot_convert;
+
 #define DEF_BLOCKSIZE 1024
 #define NODQUOT (struct dquot *)NULL
 
+#include "pot.h"
+
 #ifndef lint
-static char RCS_checkquota[] = "$Id: quotacheck.c,v 2.14 1996/11/17 16:59:46 mvw Exp mvw $";
+static char RCS_checkquota[] = "$Id: quotacheck.c,v 2.18 2000/09/05 18:47:15 mvw Exp mvw $";
 #endif
 
 struct dquot {
@@ -54,8 +61,8 @@ struct dquot {
 
 struct dlinks {
    ino_t i_num;
-   __u32 id;
-   __u32 blocks;
+   u_int32_t id;
+   u_int32_t blocks;
    struct dlinks *next;
 };
 
@@ -64,16 +71,15 @@ struct dirs {
    struct dirs *next;
 };
 
-void add_to_quota(int, ino_t, uid_t, gid_t, umode_t, nlink_t, __u32);
-void store_dlinks(int, uid_t, gid_t, ino_t, __u32);
-void dump_to_file(char *, char *, int);
+void add_to_quota(int, ino_t, uid_t, gid_t, u_int32_t, nlink_t, u_int32_t);
+void store_dlinks(int, uid_t, gid_t, ino_t, u_int32_t);
+void dump_to_file(const char *, char *, int);
 void remove_list(void);
-void scan_dir(char *);
-void scan_fs(char *);
+void scan_dir(const char *);
 void abort_now(const char *, const char *, ...);
-void add_dlinks(__u32 *, __u32 *, int, int);
+void add_dlinks(u_int32_t *, u_int32_t *, int, int);
 #if defined(EXT2_DIRECT)
-void ext2_direct_scan(char *device);
+void ext2_direct_scan(const char *device);
 #endif
 
 static char bits[] = "|/-\\";
@@ -82,8 +88,8 @@ static char bits[] = "|/-\\";
 dev_t cur_dev;
 char dflag = 0, vflag = 0;
 char check_usr, check_grp;
-__u32 files_done, dirs_done;
-__u32 highestid[MAXQUOTAS];
+u_int32_t files_done, dirs_done;
+u_int32_t highestid[MAXQUOTAS];
 char *quotatypes[] = INITQFNAMES;
 char log_buf[16384]; /* for dflag stderr buffering */
 #ifdef DEBUG_MALLOC
@@ -107,7 +113,7 @@ void *xmalloc(size_t size)
 #endif
    ptr = malloc(size);
    if (ptr == (void *)NULL)
-      abort_now("xmalloc", "Virtual memory exhausted\n");
+      abort_now("xmalloc", _("Virtual memory exhausted\n"));
    memset(ptr, 0, size);
    return(ptr);
 }
@@ -142,7 +148,7 @@ static inline struct dquot *add_dquot(int id, int type)
    register struct dquot *lptr;
 
    if (dflag)
-      fprintf(stderr, "Adding dquot structure type %s for %d\n",
+      fprintf(stderr, _("Adding dquot structure type %s for %d\n"),
 	      quotatypes[type], id);
 
    lptr = (struct dquot *)xmalloc(sizeof(struct dquot));
@@ -186,8 +192,8 @@ static inline void blit()
 
 void usage()
 {
-   fputs("Usage:\n\tquotacheck [-g] [-u] [-R] [-vd] -a\n", stderr);
-   fputs("\tquotacheck [-g] [-u] [-vd] filesys ...\n", stderr);
+   fputs(_("Usage:\n\tquotacheck [-g] [-u] [-R] [-vd] -a\n"), stderr);
+   fputs(_("\tquotacheck [-g] [-u] [-vd] filesys ...\n"), stderr);
    exit(1);
 }
 
@@ -200,7 +206,12 @@ int main(int argc, char **argv)
    unsigned done;
    int argnum;
    char *usr_qfnp, *grp_qfnp;
-   register struct mntent *mnt;
+   struct mntent *mnt;
+   const char *mnt_fsname;
+   const char *mnt_fslabel;
+
+   DQUOT_CONVERT_INIT();
+   gettexton();    
 
    while ((ch = getopt(argc, argv, "avugdR")) != EOF) {
       switch (ch) {
@@ -242,6 +253,12 @@ int main(int argc, char **argv)
 
    fp = setmntent(MNTTAB, "r");
    while ((mnt = getmntent(fp)) != (struct mntent *) NULL) {
+      mnt_fsname = get_device_name(mnt->mnt_fsname);
+      if ((mnt_fslabel = strchr(mnt->mnt_fsname, '=')) != NULL)
+         mnt_fslabel++;
+      else
+         mnt_fslabel = mnt_fsname;
+
       check_usr = check_grp = 0;
 
       if (!aflag) {
@@ -251,8 +268,7 @@ int main(int argc, char **argv)
 	} else
 	  continue;
       } else {
-	if (hasmntopt(mnt, MNTOPT_NOAUTO) ||
-	    hasmntopt(mnt, MNTOPT_NOQUOTA))
+	if (hasmntopt(mnt, MNTOPT_NOQUOTA))
 	  continue;
         else
 	   if (aflag && Rflag && !strcmp(mnt->mnt_dir, "/"))
@@ -265,13 +281,13 @@ int main(int argc, char **argv)
          check_usr++;
       if (check_usr || check_grp) {
 	 if ((lstat(mnt->mnt_dir, &st)) == -1) {
-	   fprintf(stderr, "%s: not found\n", mnt->mnt_dir);
+	   fprintf(stderr, _("%s: not found\n"), mnt->mnt_dir);
 	   perror("lstat");
            continue;
 	 }
 
 	 if (vflag)
-	   fprintf(stdout,"Scanning %s [%s] ", mnt->mnt_fsname, mnt->mnt_dir);
+	   fprintf(stdout,_("Scanning %s [%s] "), mnt_fslabel, mnt->mnt_dir);
 
 	 if (S_ISDIR(st.st_mode)) {
            cur_dev = st.st_dev;
@@ -284,8 +300,9 @@ int main(int argc, char **argv)
                            st.st_nlink, st.st_blocks);
 
 #if defined(EXT2_DIRECT)
-           if (!strcmp(mnt->mnt_type, MNTTYPE_EXT2)) {
-              ext2_direct_scan(mnt->mnt_fsname);
+           if (!strcmp(mnt->mnt_type, MNTTYPE_EXT2) ||
+               !strcmp(mnt->mnt_type, MNTTYPE_EXT3)) {
+              ext2_direct_scan(mnt_fsname);
            } else if (mnt->mnt_dir) {
 #else
            if (mnt->mnt_dir) {
@@ -294,19 +311,19 @@ int main(int argc, char **argv)
            }
 	   dirs_done++;
 	   if (vflag)
-	      fputs("done\n", stderr);
+	      fputs(_("done\n"), stderr);
            if (vflag || dflag)
-              fprintf(stderr, "Checked %d directories and %d files\n",
+              fprintf(stderr, _("Checked %d directories and %d files\n"),
                       dirs_done, files_done);
 	 } else {
-	   fprintf(stderr, "%s: not a directory\n", mnt->mnt_dir);
+	   fprintf(stderr, _("%s: not a directory\n"), mnt->mnt_dir);
 	   exit(0);
 	 }
 
          if (check_usr)
-            dump_to_file(mnt->mnt_fsname, usr_qfnp, USRQUOTA);
+            dump_to_file(mnt_fsname, usr_qfnp, USRQUOTA);
          if (check_grp)
-            dump_to_file(mnt->mnt_fsname, grp_qfnp, GRPQUOTA);
+            dump_to_file(mnt_fsname, grp_qfnp, GRPQUOTA);
          remove_list();
       }
    }
@@ -314,17 +331,17 @@ int main(int argc, char **argv)
 
    for (cnt = 0; cnt < argc; cnt++)
       if ((done & (1 << cnt)) == 0)
-         fprintf(stderr, "%s not found in fstab\n", argv[cnt]);
+         fprintf(stderr, _("%s not found in fstab\n"), argv[cnt]);
 
 #ifdef DEBUG_MALLOC
-   fprintf (stderr, "Allocated %d bytes memory\nFree'd %d bytes\nLost %d bytes\n", malloc_mem,
-	free_mem, malloc_mem - free_mem);
+   fprintf (stderr, _("Allocated %d bytes memory\nFree'd %d bytes\nLost %d bytes\n"), 
+               malloc_mem, free_mem, malloc_mem - free_mem);
 #endif
    exit(0);
 }
 
 #if defined(EXT2_DIRECT)
-void ext2_direct_scan(char *device)
+void ext2_direct_scan(const char *device)
 {
    ino_t i_num;
    ext2_filsys fs;
@@ -336,34 +353,34 @@ void ext2_direct_scan(char *device)
    ext2fs_inode_bitmap inode_dir_map;
 
    if ((error = ext2fs_open(device, 0, 0, 0, unix_io_manager, &fs))) {
-      fprintf(stderr, "quotacheck: error while opening %s\n", device);
+      fprintf(stderr, _("quotacheck: error while opening %s\n"), device);
       exit (0);
    }
 
-   if ((error = ext2fs_allocate_inode_bitmap(fs, "in-use inode map", &inode_used_map))) {
-      fprintf (stderr, "quotacheck: error while allocating inode file bitmap\n");
+   if ((error = ext2fs_allocate_inode_bitmap(fs, _("in-use inode map"), &inode_used_map))) {
+      fprintf (stderr, _("quotacheck: error while allocating inode file bitmap\n"));
       exit (0);
    }
 
-   if ((error = ext2fs_allocate_inode_bitmap(fs, "directory inode map", &inode_dir_map))) {
-      fprintf (stderr, "quotacheck: error while allocating inode directory bitmap\n");
+   if ((error = ext2fs_allocate_inode_bitmap(fs, _("directory inode map"), &inode_dir_map))) {
+      fprintf (stderr, _("quotacheck: error while allocating inode directory bitmap\n"));
       exit (0);
    }
 
    if ((error = ext2fs_open_inode_scan(fs, inode_buffer_blocks, &scan))) {
-      fprintf(stderr, "quotacheck: error while opening inode scan\n");
+      fprintf(stderr, _("quotacheck: error while opening inode scan\n"));
       exit (0);
    }
 
    if ((error = ext2fs_get_next_inode(scan, &i_num, &inode))) {
-      fprintf(stderr, "quotacheck: error while starting inode scan\n");
+      fprintf(stderr, _("quotacheck: error while starting inode scan\n"));
       exit (0);
    }
 
    while (i_num) {
       if (inode.i_links_count) {
          if (dflag)
-	    printf ("Found i_num %ld\n", i_num);
+	    printf (_("Found i_num %ld\n"), i_num);
          if (vflag)
 	    blit();
          if (check_usr)
@@ -379,7 +396,7 @@ void ext2_direct_scan(char *device)
       }
 
       if ((error = ext2fs_get_next_inode(scan, &i_num, &inode))) {
-         fprintf (stderr, "Something weird while scanning\n");
+         fprintf (stderr, _("Something weird while scanning\n"));
          exit (0);
       }
    }
@@ -391,7 +408,7 @@ void ext2_direct_scan(char *device)
  * of the files to the appropriate quotas. When we find a dir we recursivly call
  * ourself to scan that dir.
  */
-void scan_dir(char *pathname)
+void scan_dir(const char *pathname)
 {
    struct dirs *dir_stack = {(struct dirs *)NULL};
    struct dirs *new_dir;
@@ -410,8 +427,8 @@ void scan_dir(char *pathname)
          blit();
 
       if ((lstat(de->d_name, &st)) == -1) {
-	 fprintf(stderr, "Hmm, file `%s/%s' not found\n", pathname, de->d_name);
-         fputs("Guess you'd better run fsck first !\nexiting...\n", stderr);
+	 fprintf(stderr, _("Hmm, file `%s/%s' not found\n"), pathname, de->d_name);
+         fputs(_("Guess you'd better run fsck first !\nexiting...\n"), stderr);
          perror("lstat");
          exit(1);
       }
@@ -438,7 +455,7 @@ void scan_dir(char *pathname)
          dir_stack = new_dir;
       } else {
 	 if (dflag)
-	    fprintf(stderr, "\tAdding %s size %d ino %d links %d\n", de->d_name,
+	    fprintf(stderr, _("\tAdding %s size %d ino %d links %d\n"), de->d_name,
 		    st.st_size, st.st_ino, st.st_nlink);
          files_done++;
       }
@@ -449,12 +466,12 @@ void scan_dir(char *pathname)
     * Traverse the directory stack, and check it.
     */
    if (dflag)
-      fputs("Scanning stored directories from directory stack\n", stderr);
+      fputs(_("Scanning stored directories from directory stack\n"), stderr);
    while (dir_stack != (struct dirs *)NULL) {
       new_dir = dir_stack;
       dir_stack = dir_stack->next;
       if (dflag)
-         fprintf(stderr, "popd %s\nEntering directory %s\n",
+         fprintf(stderr, _("popd %s\nEntering directory %s\n"),
                  new_dir->dir_name, new_dir->dir_name);
       scan_dir(new_dir->dir_name);
       dirs_done++;
@@ -465,19 +482,19 @@ void scan_dir(char *pathname)
       free(new_dir);
    }
    if (dflag)
-      fprintf(stderr, "Leaving %s\n", pathname);
+      fprintf(stderr, _("Leaving %s\n"), pathname);
 }
     
 /*
  * Store a hardlinked file for later. Add the end we add this to a users
  * quota because we don't wanna count it more then ones.
  */
-void store_dlinks(int type, uid_t i_uid, gid_t i_gid, ino_t i_num, __u32 i_blocks)
+void store_dlinks(int type, uid_t i_uid, gid_t i_gid, ino_t i_num, u_int32_t i_blocks)
 {
    struct dlinks *lptr;
 
    if (dflag)
-      fprintf(stderr, "Adding hardlink for ino %d\n", i_num);
+      fprintf(stderr, _("Adding hardlink for ino %d\n"), i_num);
 
    for (lptr = stored_links[type]; lptr != (struct dlinks *)NULL; lptr = lptr->next)
       if (lptr->i_num == i_num)
@@ -497,8 +514,8 @@ void store_dlinks(int type, uid_t i_uid, gid_t i_gid, ino_t i_num, __u32 i_block
 /*
  * Add a number of blocks and inodes to a quota.
  */
-void add_to_quota(int type, ino_t i_num, uid_t i_uid, gid_t i_gid, umode_t i_mode,
-                  nlink_t i_nlink, __u32 i_blocks)
+void add_to_quota(int type, ino_t i_num, uid_t i_uid, gid_t i_gid, u_int32_t i_mode,
+                  nlink_t i_nlink, u_int32_t i_blocks)
 {
    int wanted;
    struct dquot *lptr;
@@ -523,7 +540,7 @@ void add_to_quota(int type, ino_t i_num, uid_t i_uid, gid_t i_gid, umode_t i_mod
 
    if ((lptr = lookup_dquot(wanted, type)) == NODQUOT)
       if ((lptr = add_dquot(wanted, type)) == NODQUOT)
-	 abort_now("add_to_quota", "Can't add dquot structure type %s for uid %d\n",
+	 abort_now("add_to_quota", _("Can't add dquot structure type %s for uid %d\n"),
 		   quotatypes[type], wanted);
 
    if (i_nlink != 1) {
@@ -548,12 +565,12 @@ void abort_now(const char *perror_mes, const char *fmt, ...)
    exit(-1);
 }
 
-void add_dlinks(__u32 * inodes, __u32 * blocks, int id, int type)
+void add_dlinks(u_int32_t * inodes, u_int32_t * blocks, int id, int type)
 {
    struct dlinks  *lptr;
 
    if (dflag)
-      fprintf(stderr, "Adding blocks from hardlinks for %s %d\n",
+      fprintf(stderr, _("Adding blocks from hardlinks for %s %d\n"),
               quotatypes[type], id);
 
    for (lptr = stored_links[type]; lptr != (struct dlinks *)NULL; lptr = lptr->next) {
@@ -608,8 +625,9 @@ void remove_list()
  * Dump the quota info that we have in memory now to the appropriate
  * quota file. We lock it during the time we update it.
  */
-void dump_to_file(char *fsname, char *quotafile, int type)
+void dump_to_file(const char *fsname, char *quotafile, int type)
 {
+   struct ondisk_dqblk od_dqblk;
    struct dqblk dq_dqb;
    struct dquot *dquot;
    struct stat st;
@@ -617,13 +635,13 @@ void dump_to_file(char *fsname, char *quotafile, int type)
    int fd, id = 0;
 
    if (vflag || dflag)
-      fprintf(stderr, "Using quotafile %s\n", quotafile);
+      fprintf(stderr, _("Using quotafile %s\n"), quotafile);
 
    if (quotactl(QCMD(Q_GETQUOTA, type), fsname, 0, (caddr_t)&dq_dqb) == 0)
       quota_enabled = 1;
 
    if ((vflag || dflag) && quota_enabled)
-      fprintf(stderr, "Updating in-core %s quotas\n", quotatypes[type]);
+      fprintf(stderr, _("Updating in-core %s quotas\n"), quotatypes[type]);
 
    if ((fd = open(quotafile, O_RDWR | O_CREAT, 0600)) < 0)
       abort_now("open", "dump_to_file(%s): ", quotafile);
@@ -636,15 +654,16 @@ void dump_to_file(char *fsname, char *quotafile, int type)
     * quotafile. Only dump new gracetimes when creating a new 
     * quotafile.
     */
-   if (read(fd, &dq_dqb, sizeof(struct dqblk)) <= 0) {
+   if (read(fd, &od_dqblk, ondisk_dqblk_size) <= 0) {
       memset((caddr_t *)&dq_dqb, 0, sizeof(struct dqblk));
       dq_dqb.dqb_btime = MAX_DQ_TIME;
       dq_dqb.dqb_itime = MAX_IQ_TIME;
-      write(fd, &dq_dqb, sizeof(struct dqblk));
+      CONVERT_TO_KERNEL_DQUOT(&od_dqblk, &dq_dqb);
+      write(fd, &od_dqblk, ondisk_dqblk_size);
    }
 
    if (fstat(fd, &st) == 0) {
-      max_id = (st.st_size / sizeof(struct dqblk)) - 1;
+      max_id = (st.st_size / ondisk_dqblk_size) - 1;
       if (max_id < highestid[type])
          max_id = highestid[type];
    } else
@@ -653,18 +672,19 @@ void dump_to_file(char *fsname, char *quotafile, int type)
    for (id = 0; id <= max_id; id++) {
       memset((caddr_t)&dq_dqb, 0, sizeof(struct dqblk));
 
-      if (lseek(fd, dqoff(id), SEEK_SET) == dqoff(id))
-         read(fd, &dq_dqb, sizeof(struct dqblk));
-
+      if (lseek(fd, dqoff(id), SEEK_SET) == dqoff(id)) {
+         read(fd, &od_dqblk, ondisk_dqblk_size);
+	 CONVERT_TO_USER_DQUOT(&dq_dqb, &od_dqblk);
+      }
       if ((dquot = lookup_dquot(id, type)) != NODQUOT) {
          dq_curinodes = dquot->dq_curinodes;
          dq_curblocks = dquot->dq_curblocks;
          if (dflag)
-            fprintf(stderr, "%s %d: curinodes: %d curblocks: %d without hardlinks\n",
+            fprintf(stderr, _("%s %d: curinodes: %d curblocks: %d without hardlinks\n"),
                     quotatypes[type], id, dq_curinodes, dq_curblocks);
-         add_dlinks((__u32 *)&dq_curinodes, (__u32 *)&dq_curblocks, id, type);
+         add_dlinks((u_int32_t *)&dq_curinodes, (u_int32_t *)&dq_curblocks, id, type);
          if (dflag)
-            fprintf(stderr, "%s %d: curinodes: %d curblocks: %d with hardlinks\n",
+            fprintf(stderr, _("%s %d: curinodes: %d curblocks: %d with hardlinks\n"),
                     quotatypes[type], id, dq_curinodes, dq_curblocks);
       } else {
          if (dq_curinodes == 0 && dq_curblocks == 0)
@@ -681,8 +701,10 @@ void dump_to_file(char *fsname, char *quotafile, int type)
          if (quotactl(QCMD(Q_SETUSE, type), fsname, id, (caddr_t)&dq_dqb) == 0)
             continue;
 
-      if (lseek(fd, dqoff(id), SEEK_SET) == dqoff(id))
-         write(fd, &dq_dqb, sizeof(struct dqblk));
+      if (lseek(fd, dqoff(id), SEEK_SET) == dqoff(id)) {
+         CONVERT_TO_KERNEL_DQUOT(&od_dqblk, &dq_dqb);
+         write(fd, &od_dqblk, ondisk_dqblk_size);
+      }
    }
    flock(fd, LOCK_UN);
    close(fd);
